// file: ChanController.js
import {AbstractPostFormView} from '../Views/AbstractPostFormView.js'
import {ChanPageView} from '../Views/ChanPageView.js'
import {NewThreadFormView} from '../Views/NewThreadFormView.js'
import {PostSubmitter} from '../Submitters/PostSubmitter.js'
import * as locationService from '../services/location_service';
import * as repo from "../repository";
import * as urlService from "../services/url_service";
import {config} from "../config";
import * as settings from "../services/settings_service";
import {TitleView} from "../Views/TitleView";
import {getThreadPageNumber} from "../services/page_number_service";

export function loadChan() {
    TitleView.set("pfpmd chan");
    document.body.id = "chan";
    document.getElementById("chan-menu-item").classList.add("active");
    document.getElementById("chan-body").content.appendAfter(document.getElementById("menu"));

    new AbstractPostFormView(publishParts);

    const postId = locationService.getPostId();
    if (postId) {
        loadPostPage(postId);
        return;
    }
    const threadId = locationService.getSelectedThreadId();
    const boardName = locationService.getSelectedBoardName();
    const pageNumber = locationService.getPage();

    if (threadId) {
        const threadPagePromise = repo.getThreadPage(threadId, pageNumber);

        ChanPageView.loadThread(threadPagePromise).then(locationService.activateTarget);
    } else {
        const warpPagePromise = repo.getWarpPage(boardName, pageNumber).then(r => {
            r.data.shellThreads = r.data.shellThreads.filter(
                st => !settings.isThreadIgnored(st.thread, st.op)
            ).map(st => {
                st.posts = st.posts.filter(p => !settings.isPostIgnored(p));
                return st;
            });
            return r;
        });
        const isBoardDefined = !!boardName;
        ChanPageView.loadWarp(warpPagePromise, isBoardDefined);
        new NewThreadFormView();
    }
    // chan.removeQuotePreviewsOnClick();
}

function loadPostPage(postId) {
    const threadPagePromise = repo.getPost(postId, true).then(post => {
        const pageNumber = getThreadPageNumber(post.index);

        window.history.replaceState('', '', urlService.getThreadUrl(
            post.thread.board.name,
            post.thread.threadId,
            pageNumber,
            post.postId,
        ));

        return repo.getThreadPage(
            post.thread.threadId,
            pageNumber,
        ).then(r => {
            r.data.posts = r.data.posts.filter(p => !settings.isPostIgnored(p));
            return r;
        });
    });

    ChanPageView.loadThread(threadPagePromise).then(locationService.activateTarget);
}

function publishParts(parts, params) {
    PostSubmitter.publishParts(parts, params).then(newThreadId => {
        openThreadBottom(params.boardName, newThreadId);
    });
}

function openThreadBottom(boardName, threadId) {
    const url = urlService.getThreadUrl(boardName, threadId);

    if (location.pathname + location.search === url) {
        location.hash = "bottom";
        location.reload();
    } else {
        location.href = `${url}#bottom`;
    }
}
