// file: AbstractPostFormView.js
import {PostBodyView} from '../Views/PostBodyView.js';
import * as settings from '../services/settings_service';
import {showNotification} from "./notifications/notification_v";
import {config} from "../config";

let instance;
const AbstractPostFormView = function (publishParts) {
    const that = this;
    if (instance) {
        throw new Error("There cannot be more than one AbstractPostFormView");
    }
    instance = that;
    AbstractPostFormView.getInstance = function () {
        if (!instance) {
            throw new Error("There is no AbstractPostFormView instance");
        }
        return instance;
    };

    if (!publishParts) throw new Error("`publishParts` must be set");

    that.BasePostFormView = function (textboxView) {
        const base = this;
        if (!textboxView) throw new Error("textboxView must be set");

        base.publishParts = (parts, params) => {
            params.timeout = settings.getSubmissionTimeout();
            params.onTimeout = onSubmissionTimeout;
            params.onError = onSubmissionFailure;

            publishParts(parts, params);
        };

        base.insertPostId = (postId) => {
            const postIdText = PostBodyView.getPostIdText(postId);
            textboxView.insertText(postIdText);
        };

        base.insertText = (text) => {
            textboxView.insertText(text);
        };

        base.getTextBeforeSelectionEnd = textboxView.getTextBeforeSelectionEnd;

        base.getOnPartIsSentFunction = (numberOfParts) => {
            textboxView.clear();

            return function onPartIsSent(currentNumberOfParts) {
                const numberOfPartsToBeSent = currentNumberOfParts - 1;
                if (numberOfPartsToBeSent === 0) {
                    textboxView.clearPlaceholder();
                    return;
                }
                const numberOfSentParts = numberOfParts - numberOfPartsToBeSent;
                textboxView.setPlaceholder(
                    `${numberOfSentParts}/${numberOfParts} is sent`
                );
            };
        };

        function onSubmissionTimeout() {
            textboxView.setPlaceholder(settings.getSubmissionTimeoutMsg());
        }

        function onSubmissionFailure(e) {
            if (e.errorCode === 'E0401') {
                showNotification(config.notification.type.error, `${e.message}. Please log in.`);
                return;
            }

            textboxView.setPlaceholder(settings.getSubmissionFailureMsg());
        }
    };
};

export {AbstractPostFormView}
