// file: ChanPostView.js
import {ChanPostBaseView} from '../BaseViews/ChanPostBaseView.js'
import {PostBodyView} from './PostBodyView.js'
import {PostReferenceView} from './PostReferenceView.js'
import {toShortcutUuid} from "../services/uuid_service";
import {formatDate} from "../utils";
import * as repo from "../repository";
import {UpdatedHiddenByOp} from "../models";
import {showNotification} from "./notifications/notification_v";
import {config} from "../config";

const ChanPostView = function (post, isPostJoined, skipAddingId, isOp) {
    const that = this;
    const base = new ChanPostBaseView(isPostJoined);

    base.postDivE.className = "post";
    if (!skipAddingId) base.postDivE.id = `p${post.postId}`;
    base.postheadDivE.className = "posthead";
    base.postheadDivE.insertTextBefore(" ", base.nameSpanE);

    base.postIdSpanE.className = "post-id";
    base.postIdSpanE.style.color = post.fg.toHSL();
    base.postIdSpanE.style.backgroundColor = post.bg.toHSL();
    base.postIdSpanE.setAttribute("data-post-id", post.postId);
    base.postIdSpanE.appendText(toShortcutUuid(post.postId));

    base.nameSpanE.classList.add("name");
    base.nameSpanE.appendText(post.user.nickname);
    if (post.user.banned) base.nameSpanE.classList.add('banned');
    base.postbodyDivE.className = "postbody";

    base.postheadDivE.appendText(` : ${formatDate(post.postDate)}`);
    base.postbodyDivE.innerHTML = PostBodyView.getFormattedText(post.text);

    if (isPostJoined) {
        base.postheadDivE.hide();
        base.postDivE.classList.add("joined");
        base.postbodyDivE.classList.add("joined");

        base.replyDivE.classList.add("reply-to-post");
        base.replyDivE.appendText("");

        base.replyDivE.title = `${post.postId} - ${formatDate(post.postDate)}`;
    }

    base.hideByOpBtnE.classList.add("hide_by_op_btn");
    base.unhideByOpBtnE.classList.add("unhide_by_op_btn");
    base.hideByOpBtnE.title = `Hide for all (you have to be OP)`;
    base.unhideByOpBtnE.title = `Unhide for all (you have to be OP)`;

    function hideByOp() {
        base.postDivE.classList.add("hidden_by_op");
        base.hideByOpBtnE.hide();
        base.unhideByOpBtnE.show();
    }

    function unhideByOp() {
        base.postDivE.classList.remove("hidden_by_op");
        base.hideByOpBtnE.show();
        base.unhideByOpBtnE.hide();
    }

    if (post.hiddenByOp) {
        hideByOp();
    } else {
        unhideByOp();
    }

    if (isOp || post.thread.threadId === config.chat.threadId) {
        base.hideByOpBtnE.hide();
    }

    that.appendTo = base.appendTo;

    that.setOnClickPostIdEventHandler = (eventHandler) => {
        if (isPostJoined) {
            base.replyDivE.onclick = eventHandler;
        } else {
            base.postIdSpanE.onclick = eventHandler;
        }
    };

    const referenceSpansEs = base.postbodyDivE.getElementsByClassName("msgref");
    referenceSpansEs.forEach(referenceSpanE => {
        const postReferenceView = new PostReferenceView();
        postReferenceView.initialize(referenceSpanE);
        postReferenceView.setMouseEvents();
    });

    base.unhideByOpBtnE.addEventListener('click', e => {
        repo.updateHiddenByOp(new UpdatedHiddenByOp(null, post.postId, false)).then(_ => {
            unhideByOp();
            showNotification(config.notification.type.success, `The post was unhidden.`);
        }).catch(e => {
            if (e.message === "Invalid clientId") {
                showNotification(config.notification.type.error,
                    `You're not OP of this thread: invalid clientId`, 5000);
            } else {
                showNotification(config.notification.type.error,
                    `Failed to unhide the post: ${e.message}`);
            }
        });
    }, false);

    base.hideByOpBtnE.addEventListener('click', e => {
        repo.updateHiddenByOp(new UpdatedHiddenByOp(null, post.postId, true)).then(_ => {
            hideByOp();
            showNotification(config.notification.type.success, `The post was hidden.`);
        }).catch(e => {
            if (e.message === "Invalid clientId") {
                showNotification(config.notification.type.error,
                    `You're not OP of this thread: invalid clientId`, 5000);
            } else {
                showNotification(config.notification.type.error,
                    `Failed to hide the post: ${e.message}`);
            }
        });
    }, false);
};

export {ChanPostView}
