// file: ChatPageView.js
import {TitleView} from './TitleView.js'
import {ChatFrameView} from './ChatFrameView.js'
import {SettingsView} from './SettingsView.js'
import {HelpView} from './HelpView.js'
import * as repo from "../repository";
import {initAliveTabsService} from "../services/alive_tabs_service";
import {ExportV} from "./export_v";
import {ImportV} from "./import_v";

const ChatPageView = (function () {
    const that = {};

    that.start = (recentPostsPromise) => {
        function setTitle() {
            TitleView.set("pfpmd chat");
        }

        setTitle();

        document.body.onclick = setTitle;
        document.addEventListener("keydown", setTitle, false);

        recentPostsPromise.then(posts => {
            const chatFrameView = new ChatFrameView(posts);

            const chatSettingsView = new SettingsView();
            chatSettingsView.setFunctionToCallAfterSave(
                chatFrameView.reloadChatFrame
            );

            new HelpView();
            new ExportV();
            new ImportV();

            initAliveTabsService();
            repo.subscribe();

            repo.addConnectedEventListener(chatFrameView.onConnected);
            repo.addPostsListener((_, post) => chatFrameView.onServerEventReceived([post]));
            repo.addErrorEventListener(chatFrameView.onServerEventError);
        });
    };

    return that;
})();

export {ChatPageView}
