// file: ImageView.js
import {ImageFailView} from './ImageFailView.js'
// done
const ImageView = function (maxContentLength, maxImgHeight) {

    let onImageLoadEventHandler;
    this.setOnImageLoadEventHandler = (eventHandler) => {
        onImageLoadEventHandler = eventHandler;
    };

    this.insertImage = (link) => {
        const params = getParamsForImageXhr(link);
        const rnd = Math.random();

        params.onLoadEventName = `onImageHeadersLoad${params.url}${rnd}`;
        params.onErrorEventName = `onImageHeadersLoadError${params.url}${rnd}`;

        const eventForGmXhr = new CustomEvent("eventForGmXhr", {
            'detail': params
        });
        window.dispatchEvent(eventForGmXhr);

        addEventListener(params.onLoadEventName, (e) => {
            params.onload(e.detail);
        });
        addEventListener(params.onErrorEventName, params.onerror);
    };

    function getParamsForImageXhr(link) {
        function onImageHeadersLoad(responseHeaders) {
            const headers = responseHeaders.split('\n');
            //console.log(response, headers);
            const contentLength = getContentLength(headers);

            if (!contentLength) {
                onImageHeadersLoadError();
                return;
            }

            if (contentLength - maxContentLength < 0) {
                const imgInfoDiv = document.createElement("div");
                const img = new Image();

                img.style.maxHeight = maxImgHeight + "px";
                img.onload = (e) => {
                    imgInfoDiv.appendText(`(${this.naturalWidth}x${this.naturalHeight}, ${formatSize(contentLength)})`);

                    onImageLoadEventHandler(e);
                };
                img.src = link.href;
                link.innerHTML = "";
                link.className += " imglink";

                link.appendChild(img);
                link.appendChild(imgInfoDiv);
            }
            else {
                const tooBigView = new ImageFailView(
                    `[size is too big: ${formatSize(contentLength)}]`
                );

                link.appendText(" ");
                tooBigView.appendTo(link);
            }
        }

        function onImageHeadersLoadError() {
            const unknownSizeView = new ImageFailView("[unknown size]");
            link.appendText(" ");
            unknownSizeView.appendTo(link);
        }

        return {
            method: 'HEAD',
            url: link.href,
            onload: onImageHeadersLoad,
            onerror: onImageHeadersLoadError
        }
    }
};

export {ImageView}

function formatSize(size) {
    let res = '';
    if (size < 1024) {
        res = 'b';
    }
    else if (size >= 1024 && size < 1024 * 1024) {
        res = 'Kb';
        size /= 1024;
    }
    else if (size >= 1024 * 1024) {
        res = 'Mb';
        size /= (1024 * 1024);
    }
    size = parseFloat(size.toFixed(2));
    return String.format("{0} {1}", size, res);
}

function getContentLength(headers) {
    for (let j = 0; j < headers.length; j++) {
        if (/^Content-Length/i.test(headers[j])) {
            return parseInt(headers[j].substring('Content-Length: '.length));
        }
    }
    return undefined;
}