// file: KeepScrollBottomView.js
// done
const KeepScrollBottomView = function (element) {
    const that = this;
    element.scrollToBottom();

    const threshold = parseFloat(
        element.attributes.getNamedItem("keep-scroll-threshold").value
    );

    function isScrolledToBottom() {
        return (element.scrollHeight - element.offsetHeight) - threshold
            <= element.scrollTop;
    }

    // keep track whether the element is scrolled to the bottom or not
    // (we need to know it on a resize event and after adding new posts)
    let wasScrolledToBottom = true;
    function keepTrackOfScrollStatus() {
        wasScrolledToBottom = isScrolledToBottom(element);
    }
    element.addEventListener("scroll", keepTrackOfScrollStatus, false);

    let resizeTimeout = null;
    function keepScrollBottomOnResize() {
        if (resizeTimeout) return;
        that.keepScrollBottom();

        resizeTimeout = setTimeout(() => {
            resizeTimeout = null;
        }, 40); // 25fps
    }
    addEventListener("resize", keepScrollBottomOnResize, false);

    that.keepScrollBottom = () => {
        if (wasScrolledToBottom) element.scrollToBottom();
    }
};

export { KeepScrollBottomView }
