// file: NavigationBarView.js
// done
const NavigationBarView = (() => {
    let instance;
    return function (currentPage, numberOfRows, pageSize, getUrlForPage) {
        const that = this;
        if (instance) {
            throw new Error("There cannot be more than one navigation bar");
        }
        instance = that;
        NavigationBarView.getInstance = () => {
            if (!instance) {
                throw new Error("There is no navigation bar instance");
            }
            return instance;
        };

        const lastPageNumber = (numberOfRows === 0 ? 0 : Math.floor((numberOfRows - 1) / pageSize));
        if (currentPage === null) currentPage = lastPageNumber;

        function isLastPage(page) {
            return lastPageNumber === page;
        }

        // initialize the last page link
        const lastPageLink = document.getElementById("lastpage");
        if (lastPageNumber === 0) {
            // in this case navigation is not needed
            document.getElementById("navaigation-wrapper").style.display = "none";
            return;
        } else {
            lastPageLink.appendText(lastPageNumber);
            lastPageLink.href = getUrlForPage(lastPageNumber);
        }

        // initialize "Next" button
        const nextBtn = document.getElementById("nextbtn");
        if (currentPage !== null && !isLastPage(currentPage)) {
            const nextPage = currentPage + 1;
            nextBtn.onclick = () => {
                location.href = getUrlForPage(nextPage);
            };
        } else {
            nextBtn.style.display = "none";
            lastPageLink.parentNode.className += " currentpage";
        }

        // initialize "Previous" button and the zero page link.
        const prevBtn = document.getElementById("prevbtn");
        const zeroPageLink = document.getElementById("zeropage");
        if (currentPage !== 0) {
            const prevPage = currentPage - 1;
            prevBtn.onclick = () => {
                location.href = getUrlForPage(prevPage);
            }
        } else {
            prevBtn.style.display = "none";
            zeroPageLink.parentNode.className += " currentpage";
        }
        document.getElementById("zeropage").href = getUrlForPage(0);

        function addCurrentPageLink() {
            addPageLink(currentPage, true);
        }

        const navigationDiv = document.getElementById("navigation");
        navigationDiv.show();

        function insertBeforeLastPageLink(element) {
            navigationDiv.insertBefore(element, lastPageLink.parentNode);
        }

        function addPageLink(page, isCurrentPage) {
            const pageSpan = document.createElement("span");
            pageSpan.className = `page ${isCurrentPage ? "currentpage" : ""}`;

            const pageLink = document.createElement("a");
            pageLink.className = "page_link";
            pageLink.href = getUrlForPage(page);
            pageLink.appendChild(document.createTextNode(page));

            pageSpan.appendChild(pageLink);
            // <span class="page"><a class="page_link" href="#">0</a></span>
            insertBeforeLastPageLink(pageSpan);
        }

        function addGap() {
            const gapSpan = document.createElement("span");
            gapSpan.appendText(String.fromCharCode(8230));
            insertBeforeLastPageLink(gapSpan);
        }

        let fromPage = currentPage - 3;
        let toPage = currentPage + 3;

        if (currentPage < 4) {
            toPage += 4 - currentPage;
        }
        if (currentPage > lastPageNumber - 4) {
            fromPage -= currentPage - lastPageNumber + 4;
        }

        fromPage = fromPage < 1 ? 1 : fromPage;
        toPage = toPage >= lastPageNumber ? lastPageNumber - 1 : toPage;

        if (fromPage > 1) {
            addGap();
        }
        for (let i = fromPage; i <= toPage; i++) {
            if (i === currentPage) {
                addCurrentPageLink();
            } else {
                addPageLink(i, false);
            }
        }
        if (toPage < lastPageNumber - 1) {
            addGap();
        }
    }
})();

export {NavigationBarView}
