// file: NewThreadFormView.js
import {TextboxBaseView} from '../BaseViews/TextboxBaseView.js'
import {TextboxView} from './TextboxView.js'
import {AbstractPostFormView} from './AbstractPostFormView.js'
import {PostBodyView} from './PostBodyView.js'
import {isCtrlEnter} from "../services/shortcut_service";

const NewThreadFormView = (() => {
    let instance;
    return function () {
        const that = this;
        if (instance) {
            throw new Error("There cannot be more than one 'new thread' form");
        }
        instance = that;
        NewThreadFormView.getInstance = function () {
            if (!instance) {
                throw new Error("There is no 'new thread' form instance");
            }
            return instance;
        };

        const form = document.getElementById("createform");
        const wrapper = document.getElementById("createform-wrapper");
        const subjectInput = document.getElementById("subjectInput");
        const postText = document.getElementById("postText");

        const textboxBaseView = new TextboxBaseView(null, postText);
        const textboxView = new TextboxView(textboxBaseView);
        textboxView.addOnKeyDownEventHandler(ctrlEnter);
        textboxView.autoresize();

        const boardNameInput = document.getElementById("boardNameInput");
        const btn = document.getElementById("createNewThreadBtn");

        // by default it is hidden
        wrapper.style.display = "inherit";

        const abstractPostFormView = AbstractPostFormView.getInstance();
        const basePostFormView = new abstractPostFormView.BasePostFormView(
            textboxView
        );

        btn.onclick = createNewThread;
        btn.title = "Ctrl+Enter";

        function createNewThread() {
            if (!form.checkValidity()) return;

            const boardName = getBoardName();
            const subject = subjectInput.value;

            try {
                const parts = getParts();
                const onPartIsSent = basePostFormView.getOnPartIsSentFunction(
                    parts.length
                );

                basePostFormView.publishParts(parts, {
                    boardName: boardName,
                    subject: subject,
                    threadId: null,
                    onPartIsSent: onPartIsSent,
                });
            }
            catch (err) {
                console.log(err);
                return false;
            }
            return false;
        }

        function ctrlEnter(e) {
            if (isCtrlEnter(e)) createNewThread();
        }

        function getBoardName() {
            return boardNameInput.value.toLowerCase();
        }

        function getParts() {
            return PostBodyView.getParts(textboxView.getValue());
        }
    };
})();

export {NewThreadFormView}
