// file: ReplyFormView.js
import {ReplyFormBaseView} from '../BaseViews/ReplyFormBaseView.js'
import {TextboxView} from './TextboxView.js'
import {AbstractPostFormView} from './AbstractPostFormView.js'
import {PostBodyView} from './PostBodyView.js'
import {isCtrlEnter} from "../services/shortcut_service";
import {config} from "../config";
// done
const ReplyFormView = function (boardName, threadId) {
    const that = this;
    const base = new ReplyFormBaseView();

    base.replyForm.className = "replyform";
    base.replyForm.action = "/post";
    base.replyForm.method = "post";

    base.replyheadDiv.className = "replyhead";

    base.boardInput.name = "board";
    base.boardInput.setAttribute("type", "hidden");
    base.boardInput.value = boardName;

    base.threadInput.name = "thread";
    base.threadInput.setAttribute("type", "hidden");
    base.threadInput.value = threadId;

    base.replyButton.className = "replybtn";
    base.replyButton.type = "submit";
    base.replyButton.title = "Ctrl+Enter";
    base.replyButton.appendText("Reply");

    base.replybodyDiv.className = "replybody";

    const postTextareaView = new TextboxView(base.postTextareaBaseView);
    postTextareaView.setName("post");
    postTextareaView.setRequired();
    postTextareaView.setMaxLength(config.maxPostLength);
    postTextareaView.setId(`${threadId}tbx`);
    postTextareaView.addOnKeyDownEventHandler(ctrlEnter);
    postTextareaView.autoresize();

    const abstractPostFormView = AbstractPostFormView.getInstance();
    const basePostFormView = new abstractPostFormView.BasePostFormView(
        postTextareaView
    );
    that.insertPostId = basePostFormView.insertPostId;

    base.replyButton.onclick = reply;

    function reply() {
        if (!base.replyForm.checkValidity()) return;

        const boardName = base.boardInput.value;
        const threadId = base.threadInput.value;

        try {
            const parts = getParts();
            const onPartIsSent = basePostFormView.getOnPartIsSentFunction(
                parts.length
            );

            basePostFormView.publishParts(parts, {
                boardName: boardName,
                subject: null,
                threadId: threadId,
                onPartIsSent: onPartIsSent,
            });
        }
        catch (err) {
            console.log(err);
            return false;
        }
        return false;
    }

    function ctrlEnter(e) {
        if (isCtrlEnter(e)) reply();
    }

    function getParts() {
        return PostBodyView.getParts(postTextareaView.getValue());
    }

    this.hide = () => base.replyForm.hide();
    this.show = () => base.replyForm.show();

    that.appendTo = base.appendTo;
};

export {ReplyFormView}
