// file: SettingsView.js
import { ShadeView } from './ShadeView.js'
import * as settings from "../services/settings_service";
import {isEscape} from "../services/shortcut_service";
import {config} from "../config";
import {getOrCreateClientId, setClientId} from "../services/client_service";

const SettingsView = function () {
    const that = this;

    const modalsContainerDivE = document.getElementById("modals-container");
    const settingsDivE = document.getElementById("settings");
    const settingsButtonE = document.getElementById("settingsButton");

    const saveButtonE = document.getElementById("save");
    const cancelButtonE = document.getElementById("cancel");

    const elements = getSettingsElements();

    if (!config.showIgnoreCyrillicShitFilter) {
        elements.ignoreCyrillicInputE.getClosestByClassName("row").hide();
    }

    that.show = () => {
        ShadeView.addShade();
        setSettingsBox();
        modalsContainerDivE.show();
        settingsDivE.show();
        return false;
    };

    that.hide = () => {
        ShadeView.removeShade();
        modalsContainerDivE.hide();
        settingsDivE.hide();
    };

    that.setFunctionToCallAfterSave = (fn) => {
        that.afterSave = fn;
    };

    function saveSettings() {
        // todo: wrap everything in a form
        const validate = (elements.clientIdInputE.reportValidity || elements.clientIdInputE.checkValidity)
            .bind(elements.clientIdInputE);

        if (!validate()) return;

        settings.setMaxContentLength(elements.maxImgSizeInputE.value);
        settings.setMaxImgHeight(elements.maxImgHeightInputE.value);

        settings.setShowImgsFlag(elements.showImgsInputE.checked);
        settings.setEmbedBitchuteFlag(elements.embedBitchuteInputE.checked);
        settings.setShowOnlyChatPostsFlag(elements.showOnlyChatPostsE.checked);
        settings.setIgnoreCyrillicFlag(elements.ignoreCyrillicInputE.checked);
        settings.setEnableEndlessScrollFlag(elements.enableEndlessScrollInputE.checked);
        settings.setEnableKatexFlag(elements.enableKatexInputE.checked);
        settings.setHighlightCodeFlag(elements.highlightCodeInputE.checked);
        settings.setMaxEntropy(elements.maxEntropyInputE.value);
        settings.setRegexpFilters(elements.filtersTextareaE.value);
        settings.setColorsFilters(elements.colorsFiltersTextareaE.value);

        setClientId(elements.clientIdInputE.value);

        that.hide();
        if (that.afterSave) that.afterSave();
    }

    function setSettingsBox() {
        elements.maxImgSizeInputE.value = settings.getMaxContentLength();
        elements.maxImgHeightInputE.value = settings.getMaxImgHeight();

        elements.showImgsInputE.checked = settings.doShowImgs();
        elements.embedBitchuteInputE.checked = settings.doEmbedBitchute();
        elements.showOnlyChatPostsE.checked = settings.doShowOnlyChatPosts();
        elements.ignoreCyrillicInputE.checked = settings.doIgnoreCyrillic();
        elements.enableEndlessScrollInputE.checked = settings.doEnableEndlessScroll();
        elements.enableKatexInputE.checked = settings.doEnableKatex();
        elements.highlightCodeInputE.checked = settings.doHighlightCode();
        elements.maxEntropyInputE.value = settings.getMaxEntropy();
        elements.filtersTextareaE.value = settings.getRegexpFilters();
        elements.colorsFiltersTextareaE.value = settings.getColorsFilters();

        elements.clientIdInputE.disabled = true;
        getOrCreateClientId().then(clientId => {
            elements.clientIdInputE.disabled = false;
            elements.clientIdInputE.value = clientId;
        });
    }

    settingsButtonE.onclick = that.show;
    saveButtonE.onclick = saveSettings;
    cancelButtonE.onclick = that.hide;

    window.addEventListener("keydown", (event) => {
        if (isEscape(event) && !settingsDivE.isHidden()) {
            that.hide();
        }
    }, true);
};

export { SettingsView }

function getSettingsElements() {
    let elements = {};

    elements.maxImgSizeInputE = document.getElementById("maxImgSize");
    elements.maxImgHeightInputE = document.getElementById("maxImgHeight");

    elements.showImgsInputE = document.getElementById("showImgsP");
    elements.embedBitchuteInputE = document.getElementById("embedBitchuteP");
    elements.showOnlyChatPostsE = document.getElementById("showOnlyChatPostsP");
    elements.ignoreCyrillicInputE = document.getElementById("ignoreCyrillicP");
    elements.enableEndlessScrollInputE = document.getElementById("enableEndlessScrollP");
    elements.enableKatexInputE = document.getElementById("enableKatexP");
    elements.highlightCodeInputE = document.getElementById("highlightCodeP");
    elements.maxEntropyInputE = document.getElementById("maxEntropyP");
    elements.filtersTextareaE = document.getElementById("filtersP");
    elements.colorsFiltersTextareaE = document.getElementById("colorsFilters");

    elements.clientIdInputE = document.getElementById("clientIdP");

    return elements;

}