// file: ShellThreadView.js
import {ShellThreadBaseView} from '../BaseViews/ShellThreadBaseView.js'
import {ReplyFormView} from './ReplyFormView.js'
import {AbstractThreadView} from './AbstractThreadView.js'
import {ThreadInfoView} from './ThreadInfoView.js';

const ShellThreadView = function (shellThread, doAddBoardPartInThreadInfo) {
    const that = this;
    const base = new ShellThreadBaseView();

    base.shellThreadDiv.className = "shell-thread";

    const thread = shellThread.thread;
    const board = thread.board;

    const replyFormView = new ReplyFormView(board.name, thread.threadId);

    function appendPost(post, previousPost, isOp) {
        AbstractThreadView.appendPost(
            base.shellThreadDiv, post, previousPost, replyFormView, isOp
        );
    }

    const threadInfoView = new ThreadInfoView(thread, doAddBoardPartInThreadInfo);
    threadInfoView.appendTo(base.shellThreadDiv);

    appendPost(shellThread.op, null, true);

    const isRecentDivNeeded = (thread.postCount > 3);
    if (isRecentDivNeeded) {
        base.addRecentDiv();
        base.recentDiv.className = "recent";

        const br = base.recentDiv.insertBrBefore(base.threadLink);
        base.recentDiv.insertTextBefore("The 3 newest replies are shown below.", br);

        base.threadLink.href = threadInfoView.getThreadHrefP0();
        base.threadLink.appendText("Read this thread from the beginning.");
    }

    shellThread.posts.reduce((previousPost, post) => {
        appendPost(post, previousPost, false);
        return post;
    }, isRecentDivNeeded ? null : shellThread.op);

    replyFormView.appendTo(base.shellThreadDiv);

    that.appendTo = base.appendTo;
};

export {ShellThreadView}
