// file: TextboxView.js
// done
const TextboxView = function (base) {
    const that = this;

    base.textbox.setAttribute("type", "text");

    that.setName = (name) => base.textbox.name = name;
    that.setRequired = () => base.textbox.required = true;
    that.setId = (id) => base.textbox.id = id;
    that.getValue = () => base.textbox.value.trim();
    that.setValue = (value) => base.textbox.value = value;
    that.focus = () => base.textbox.focus();
    that.getTagName = () => base.textbox.tagName.toLowerCase();
    that.setPlaceholder = (value) => base.textbox.placeholder = value;
    that.clearPlaceholder = () => base.textbox.placeholder = "";

    that.setPlaceholderTemporarily = (value, timeout) => {
        that.setPlaceholder(value);
        setTimeout(() => that.clearPlaceholder(), timeout);
    };

    that.setMaxLength = (maxLength) => base.textbox.maxLength = maxLength;

    let onAfterAutoresize = () => {};

    that.setOnAfterAutoresizeEventHandler = (eventHandler) => {
        onAfterAutoresize = eventHandler;
    };

    function resize() {
        const tbx = base.textbox;
        const parent = tbx.parentNode;

        // prevent page jerking via fixing the height of the parent node
        const originalParentHeight = parent.style.height;
        parent.style.height = getComputedStyle(parent).height;

        if (tbx.scrollHeight === tbx.clientHeight) tbx.style.height = "";

        const newHeight = (tbx.scrollHeight + tbx.offsetHeight - tbx.clientHeight) + "px";

        tbx.style.height = newHeight;

        // now release the parent
        parent.style.height = originalParentHeight;

        onAfterAutoresize();
    }

    let isAutoresizable = false;
    that.autoresize = () => {
        isAutoresizable = true;
        base.textbox.addEventListener("input", resize);
    };
    that.enableAutofocus = () => base.textbox.autofocus = true;
    that.disableAutocomplete = () => base.textbox.autocomplete = "off";
    that.addOnKeyDownEventHandler = (eventHandler) => {
        base.textbox.addEventListener("keydown", eventHandler);
    };
    that.setOnKeyUpEventHandler = (eventHandler) => {
        base.textbox.onkeyup = eventHandler;
    };
    that.setOnFocusEventHandler = (eventHandler) => {
        base.textbox.onfocus = eventHandler;
    };

    that.clear = () => {
        base.textbox.value = "";
        if (isAutoresizable) resize();
    };

    that.insertText = (text) => {
        const oldText = base.textbox.value;
        const selectionEnd = base.textbox.selectionEnd;

        const firstPart = oldText.slice(0, base.textbox.selectionEnd);
        const secondPart = oldText.slice(base.textbox.selectionEnd);
        base.textbox.value = (firstPart + text + secondPart);

        base.textbox.selectionEnd = selectionEnd + text.length;
        base.textbox.focus();
        if (isAutoresizable) resize();
    };

    that.getTextBeforeSelectionEnd = () => {
        const tbxE = base.textbox;
        return tbxE.value.slice(0, tbxE.selectionEnd);
    };

    that.appendTo = base.appendTo;
    that.remove = base.remove;
};

export { TextboxView }
