// file: ThreadView.js
import {BoardInfoView} from './BoardInfoView.js'
import {ReplyFormView} from './ReplyFormView.js'
import {AbstractThreadView} from './AbstractThreadView.js'
import {ThreadInfoView} from './ThreadInfoView.js'
import {NavigationBarView} from './NavigationBarView.js'
import {ChanTitleView} from './ChanTitleView.js'
import {BlindView} from './BlindView.js'
import {getThreadPageNumber} from "../services/page_number_service";
import {config} from "../config";
import {isThreadIgnored} from "../services/settings_service";
import * as urlService from "../services/url_service";

const ThreadView = (threadPageResponse) => {
    const threadPage = threadPageResponse.data;
    const board = threadPage.thread.board;
    const threadId = threadPage.thread.threadId;
    const pageNumber = getThreadPageNumber(threadPageResponse.skipped, config.pageSize.general);

    BoardInfoView.set(board, true);

    const threadContainer = document.getElementById("thread");

    const replyFormView = new ReplyFormView(board.name, threadId);

    function appendPost(post, previousPost, isOp) {
        AbstractThreadView.appendPost(threadContainer, post, previousPost, replyFormView, isOp);
    }

    const threadInfoView = new ThreadInfoView(threadPage.thread, false);
    threadInfoView.appendTo(threadContainer);

    if (isThreadIgnored(threadPage.thread, threadPage.op)) {
        BlindView.hide();
        return;
    }
    appendPost(threadPage.op, null, true);

    threadPage.posts.reduce((previousPost, post) => {
        appendPost(post, previousPost);
        return post;
    }, pageNumber > 0 ? null : threadPage.op); // there is a bug here!

    function getUrlForPage(page) {
        return urlService.getThreadUrl(board.name, threadId, page);
    }

    const navigationBarView = new NavigationBarView(
        pageNumber,
        Math.max(threadPageResponse.total - 1, 0), // -1 to skip OP
        config.pageSize.general,
        getUrlForPage,
    );

    ChanTitleView.set(board.name, threadId, threadPage.thread.subject, pageNumber);

    replyFormView.appendTo(threadContainer);

    BlindView.hide();
};

export {ThreadView}
