// file: ThumbnailView.js
const ThumbnailView = (() => {
    const that = {};

    function create(url, className, src) {
        const imgE = document.createElement('img');
        imgE.alt = url;
        imgE.title = url;
        imgE.src = src;
        imgE.classList.add(className);
        imgE.referrerPolicy = "no-referrer";

        const pictureE = document.createElement('picture');
        pictureE.appendChild(imgE);
        return pictureE;
    }

    that.getYoutubeThumbnail = (url) => {
        const videoId = getYoutubeVideoId(url);
        if (!videoId) return null;
        return create(url, "youtube_thumb", `https://img.youtube.com/vi/${videoId}/mqdefault.jpg`);
    };

    function getYoutubeVideoId(url) {
        const regExp = /^https?:\/\/(?:www\.|m\.)?(?:youtube\.com|youtu\.be)\/(?:[\w\-]+\?v=|embed\/|v\/)?([\w\-]+)/;
        const match = url.match(regExp);
        return (match && match[1].length === 11) ? match[1] : null;
    }

    that.getImgurThumbnail = (url) => {
        const regExp = /^.*imgur.com\/(\w{6,7})\w?\.(\w{3})/;
        const match = regExp.exec(url);
        if (!match) return null;
        const imageId = match[1];
        const imageExt = match[2];
        const thumbId = imageId + "m";
        return create(url, "imgur_thumb", `https://i.imgur.com/${thumbId}.${imageExt}`);
    };
    that.getTwitterThumbnail = (url) => {
        const regExp = /^(.*pbs.twimg.com\/media\/[\w_-]+)(?:\.(\w{3,4})|\?format=(\w{3,4}))/;
        const match = regExp.exec(url);
        if (!match) return null;
        return create(url, "twitter_thumb", `${match[1]}.${match[2] || match[3]}:thumb`);
    };

    that.getPloudfrThumbnailV = (url) => {
        const regExp = /^https?:\/\/(?:www\.)?video.ploud.fr\/videos\/(?:watch|embed)\/([\w\-]+)/;
        return getPeerTubeThumbnailV(url, 'video.ploud.fr', 'ploudfr_thumb', regExp);
    };

    that.getLibreVideoThumbnailV = (url) => {
        const regExp = /^https?:\/\/(?:www\.)?libre.video\/videos\/(?:watch|embed)\/([\w\-]+)/;
        return getPeerTubeThumbnailV(url, 'libre.video', 'librevideo_thumb', regExp);
    };

    that.getPeertubeSocialThumbnailV = (url) => {
        const regExp = /^https?:\/\/(?:www\.)?peertube.social\/videos\/(?:watch|embed)\/([\w\-]+)/;
        return getPeerTubeThumbnailV(url, 'peertube.social', 'peertubesocial_thumb', regExp);
    };

    function getPeerTubeThumbnailV(url, domain, thumbClass, regExp) {
        const match = url.match(regExp);
        if (!match) return null;
        const videoId = match[1];
        return create(url, thumbClass, `https://${domain}/static/thumbnails/${videoId}.jpg`);
    }

    return that;
})();

export {ThumbnailView}
