// file: TypingLoggerView.js
// done
const TypingLoggerView = (() => {
    const that = {};
    const myTypingLog = [];

    that.runTypingLogger = (textboxView) => {
        const isInput = textboxView.getTagName() === "input";
        const isTextarea = textboxView.getTagName() === "textarea";
        if (!isInput && !isTextarea) return;
        let position = 0;

        // removes repeated messages in the log.
        function removeRepeated() {
            if (myTypingLog.length >= 2
                && myTypingLog[myTypingLog.length - 1]
                === myTypingLog[myTypingLog.length - 2]) {

                myTypingLog.pop();
                //console.log("repeated", position, myTypingLog);
            }
        }

        textboxView.addOnKeyDownEventHandler(e => {
            // up arrow (only for input) or alt+i
            if ((e.keyCode === 38 && isInput)
                || (e.altKey && e.keyCode === 73)) {
                if(position <= myTypingLog.length) {
                    position++;
                }
                //console.log("up", position, myTypingLog);
            }
            // down arrow (only for input) or alt+k
            else if ((e.keyCode === 40 && isInput)
                     || (e.altKey && e.keyCode === 75)) {
                if (position > 0) {
                    position--;
                 }
                //console.log("down", position, myTypingLog);
            } else {
                return true;
            }

            removeRepeated();

            const idx = myTypingLog.length - position;

            textboxView.setValue(idx < 0 || position < 1
                ? ""
                : myTypingLog[idx]);

            return false;
        });

        function refreshLog(e) {
            const idx = myTypingLog.length - position;
            if ((position === 0 || idx === -1)
                && textboxView.getValue() !== 0) {
                myTypingLog.push(textboxView.getValue());
                position = 1;
                //console.log("printable key up", position, myTypingLog);
            }
            else if (position === 1
                     && myTypingLog.length !== 0) {
                // if not "enter" was pressed
                if (e.keyCode !== 13) {
                    myTypingLog[idx] = textboxView.getValue();
                    if (textboxView.getValue().length === 0) {
                        myTypingLog.pop();
                        position = 0;
                        //console.log(myTypingLog, e.keyCode);
                    }
                }
                else {
                    position = 0;
                    removeRepeated();
                }
                //console.log("update first", position, myTypingLog);
            }
            else if (position > 1
                     && idx >= 0
                     && myTypingLog[idx] !== textboxView.getValue()) {
                myTypingLog.push(textboxView.getValue());
                position = 1;
                //console.log("update inside", position, myTypingLog);
            }
            return true;
        }

        textboxView.setOnKeyUpEventHandler(refreshLog);

        // after we inserted a message number, the input gets focused
        textboxView.setOnFocusEventHandler(refreshLog);

        // these two calls are needed for the case when we change <textarea> to <input>
        refreshLog({});
        removeRepeated();
    };

    return that;
})();

export { TypingLoggerView }
