// file: service_loader.js
import {importCss, importScript} from "../utils";

export class ServiceLoader {
    constructor(cssUrl, scriptUrl) {
        this._cssUrl = cssUrl;
        this._scriptUrl = scriptUrl;

        this._isServiceLoaded = false;
        this._isServiceEnabled = false;
        this._serviceLoadPromise = null;
    }

    get isServiceEnabled() {
        return this._isServiceEnabled;
    }

    enableService(isEnabled) {
        this._isServiceEnabled = isEnabled;

        // load it if enabled
        if (!isEnabled) {
            return Promise.resolve();
        }

        if (this._isServiceLoaded) {
            // already loaded
            return Promise.resolve();
        }

        if (this._serviceLoadPromise) {
            // it is loading at the moment
            return this._serviceLoadPromise;
        }

        this._serviceLoadPromise = this._loadService().then(() => this._isServiceLoaded = true);
        return this._serviceLoadPromise;
    }

    _loadService() {
        return Promise.all([
            importCss(this._cssUrl),
            importScript(this._scriptUrl)
        ]);
    }
}