// file: parsers.js
import {
    Board, Thread, Post, ThreadSlice, WarpSlice,
    SuccessResponse, ErrorResponse, ServerInfo, AuthInfo, Claims, ShortUserInfo
} from "./models";

export function parseBoard(obj) {
    return new Board(obj.name, obj.description);
}

export function parseThread(obj) {
    return new Thread(
        obj.threadId,
        obj.subject,
        obj.postCount,
        obj.board ? parseBoard(obj.board) : null,
    );
}

export function parsePost(obj) {
    return new Post(
        obj.postId,
        new Date(obj.postDate),
        obj.bg,
        obj.fg,
        obj.text,
        obj.hiddenByOp,
        parseShortUserInfo(obj.user),
        obj.thread ? parseThread(obj.thread) : null,
        obj.index,
    );
}

export function parseServerInfo(obj) {
    return new ServerInfo(
        obj.version,
        obj.powBits,
    );
}

export function parseThreadSlice(obj) {
    return new ThreadSlice(
        parseThread(obj.thread),
        parsePost(obj.op),
        obj.posts.map(parsePost),
    );
}

export function parseWarpSlice(obj) {
    return new WarpSlice(
        obj.board ? parseBoard(obj.board) : null,
        obj.shellThreads.map(parseThreadSlice),
    );
}

export function parseShortUserInfo(obj) {
    return new ShortUserInfo(
        obj.nickname,
        obj.banned,
    );
}

export function parseAuthInfo(obj) {
    return new AuthInfo(
        obj.accessToken,
    );
}

export function parseClaims(accessToken) {
    const obj = JSON.parse(atob(accessToken.split('.')[1]));
    return new Claims(
        obj.name,
        obj.iat,
    );
}

export function parseSuccessResponse(obj) {
    return dataParser => {
        return new SuccessResponse(
            dataParser(obj.data),
            obj.skipped,
            obj.taken,
            obj.total,
        );
    }
}

export function parseErrorResponse(obj) {
    return new ErrorResponse(
        obj.errorCode,
        obj.message,
    );
}