// file: alive_tabs_service.js
import * as lsService from "./storage/local_storage_service";
import {config} from "../config";

const tabCreatedAt = new Date().getTime();
const tabId = `tab_${tabCreatedAt}`;
let isEventSourceTab = false;
const isEventSourceListeners = [];

function heartbeat() {
    const aliveTabs = getOtherAliveTabs();
    const oldIsEventSourceTab = isEventSourceTab;

    // this tab will be used to connect to SSE if
    // there's no other connected tab at the moment and it's the oldest tab
    isEventSourceTab = (
        !aliveTabs.some(x => x.value.isEventSource)
        && aliveTabs.every(x => x.value.createdAt > tabCreatedAt)
    );

    lsService.setItem(tabId, {
        createdAt: tabCreatedAt,
        aliveAt: new Date().getTime(),
        isEventSource: isEventSourceTab,
    });

    if (isEventSourceTab !== oldIsEventSourceTab) {
        isEventSourceListeners.forEach(listener => listener(isEventSourceTab));
    }
}

export function initAliveTabsService() {
    cleanOldTabs();
    heartbeat();
    const intervalId = setInterval(() => heartbeat(), config.tabHeartbeatTimeout);

    addEventListener('beforeunload', e => {
        clearInterval(intervalId);
        lsService.removeItem(tabId);
    });
}

function getTabs() {
    const tabs = lsService.getItems(/^tab_/);
    // const now = new Date().getTime();
    // console.log(tabs.map(t => {
    //     return {tab: t.key, at: t.value.aliveAt, dur: (now - t.value.aliveAt)};
    // }));
    return tabs;
}

function getAliveTabs() {
    const now = new Date().getTime();

    return getTabs().filter(
        x => now - x.value.aliveAt < config.tabHeartbeatTimeout * 4
    );
}

function getOtherAliveTabs() {
    return getAliveTabs().filter(x => x.key !== tabId);
}

function cleanOldTabs() {
    const now = new Date().getTime();

    getTabs().filter(
        x => now - x.value.aliveAt >= config.tabHeartbeatTimeout * 4
    ).forEach(x => lsService.removeItem(x.key));
}

export function isCurrentTabEventSource() {
    return isEventSourceTab;
}

export function addIsEventSourceListener(listener) {
    isEventSourceListeners.push(listener);
}
