// file: location_service.js
import {config} from "../config";
import {parsePostReference, parseUuid} from "./text_parsing_service";

export function getPathName() {
    return location.pathname.substring(config.basePath.length);
}

function getPathParts() {
    return getPathName().split('/');
}

export function getSelectedBoardName() {
    const name = getPathParts()[1].toLowerCase();
    return /^[a-z]{1,3}$/.test(name) ? name : null;
}

export function getSelectedThreadId() {
    if (!getSelectedBoardName()) return null;

    return getThreadId();
}

export function getThreadId() {
    const threadId = parseUuid(getPathParts()[2]);
    return threadId ? threadId.match : null;
}


export function getPage() {
    const pageNumber = parseInt(getQueryStrings().p);
    return isNaN(pageNumber) ? null : pageNumber;
}

export function getPostId() {
    const match = /(@.+)/.exec(getPathName());
    if (!match) return null;
    return parsePostReference(match[1]).match;
}

export function activateTarget() {
    if (location.hash) {
        // noinspection SillyAssignmentJS
        location.hash = location.hash;
    }
}

function getQueryStrings() {
    let result = {};
    location.search.substring(1).split('&').forEach(x => {
        const idx = x.indexOf('=');
        if (idx > 0) {
            result[decodeURIComponent(x.substring(0, idx))]
                = decodeURIComponent(x.substring(idx + 1));
        } else {
            result[decodeURIComponent(x)] = null;
        }
    });
    return result;
}