// file: post_cache_service.js

// key: postId, value: PostModel
import {toSimpleUuid} from "./uuid_service";

const cachedPosts = {};

export function get(postId) {
    // console.log(cachedPosts);
    return cachedPosts[toSimpleUuid(postId)];
}

export function saveForRecentPosts(recentPosts) {
    saveMany(recentPosts.posts);
    return recentPosts;
}

export function saveForThreadSlice(threadSlice) {
    save(threadSlice.op);
    saveMany(threadSlice.posts);
    save(threadSlice.prevPost);
    return threadSlice;
}

export function saveForWarpSlice(warpSlice) {
    warpSlice.shellThreads.forEach(saveForThreadSlice);
    return warpSlice;
}

export function save(post) {
    if (!post) return post;

    // do not cache if thread or board not available
    if (!post.thread || !post.thread.board) return post;

    cachedPosts[toSimpleUuid(post.postId)] = post;
    return post;
}

export function saveMany(posts) {
    posts.forEach(save);
    return posts;
}
