// file: settings_service.js
import * as lsService from "./storage/local_storage_service";
import {config} from "../config";
import {containCyrillic, getShannonEntropy, parseColorsFilters, parseRegexpFilters} from "./filter_service";

export function getMaxContentLength() {
    return lsService.getItem("MaxContentLength", config.defaultSettings.maxImageContentLength);
}

export function getMaxImgHeight() {
    return lsService.getItem("MaxImgHeight", config.defaultSettings.maxImageHeight);
}

export function doShowImgs() {
    return lsService.getItem("ShowImgsP", config.defaultSettings.showImages);
}

export function doEmbedBitchute() {
    return lsService.getItem("EmbedBitchuteP", config.defaultSettings.embedBitchute);
}

export function doIgnoreCyrillic() {
    return lsService.getItem("IgnoreCyrillicP", config.defaultSettings.ignoreCyrillicShit);
}

export function doEnableKatex() {
    return lsService.getItem("enableKatexP", config.defaultSettings.isKatexEnabled);
}

export function doHighlightCode() {
    return lsService.getItem("highlightCodeP", config.defaultSettings.highlightCode);
}

export function getMaxEntropy() {
    return lsService.getItem("maxEntropyP", config.defaultSettings.maxEntropy);
}

export function getRegexpFilters() {
    // todo: rename filtersP to regexpFilters, but be aware that some clients already use filtersP!
    return lsService.getItem("filtersP", config.defaultSettings.regexpFilters);
}

export function getColorsFilters() {
    return lsService.getItem("colorsFilters", config.defaultSettings.colorsFilters);
}


// chat specific
export function doShowOnlyChatPosts() {
    return lsService.getItem("showOnlyChatPostsP", config.defaultSettings.showOnlyChatPostsInChat);
}

export function doEnableEndlessScroll() {
    return lsService.getItem("enableEndlessScrollP", config.defaultSettings.enableEndlessScroll);
}


export function setMaxContentLength(value) {
    lsService.setItem("MaxContentLength", value);
}

export function setMaxImgHeight(value) {
    lsService.setItem("MaxImgHeight", value);
}

export function setShowImgsFlag(value) {
    lsService.setItem("ShowImgsP", value);
}

export function setEmbedBitchuteFlag(value) {
    lsService.setItem("EmbedBitchuteP", value);
}

export function setIgnoreCyrillicFlag(value) {
    lsService.setItem("IgnoreCyrillicP", value);
}

export function setEnableKatexFlag(value) {
    return lsService.setItem("enableKatexP", value);
}

export function setHighlightCodeFlag(value) {
    return lsService.setItem("highlightCodeP", value);
}

export function setMaxEntropy(value) {
    return lsService.setItem("maxEntropyP", value || 0);
}

export function setRegexpFilters(value) {
    return lsService.setItem("filtersP", value || "");
}

export function setColorsFilters(value) {
    return lsService.setItem("colorsFilters", value || "");
}

// chat specific
export function setShowOnlyChatPostsFlag(value) {
    lsService.setItem("showOnlyChatPostsP", value);
}

export function setEnableEndlessScrollFlag(value) {
    lsService.setItem("enableEndlessScrollP", value);
}


export function getSubmissionTimeout() {
    return config.submissionTimeout;
}

export function getSubmissionTimeoutMsg() {
    return "It takes too long to send the message. "
        + "Check your connection.";
}

export function getSubmissionFailureMsg() {
    return "Message failed to send. "
        + "Check your connection.";
}


export function isPostIgnored(post) {
    if (isIgnored(post.text)) return true;

    return parseColorsFilters(getColorsFilters())
        .some(({bg, fg}) => post.bg === bg && post.fg === fg);
}

export function isThreadIgnored(thread, op) {
    return isPostIgnored(op) || isIgnored(thread.subject);
}

function isIgnored(text) {
    if (doIgnoreCyrillic() && containCyrillic(text)) {
        return true;
    }

    const maxEntropy = getMaxEntropy();
    let entropy;
    if (maxEntropy > 0 && (entropy = getShannonEntropy(text)) > maxEntropy) {
        console.debug("Ignored. Entropy:", entropy, text.substring(0, 500));
        return true;
    }

    return parseRegexpFilters(getRegexpFilters()).some(re => re.test(text));
}
