// file: text_parsing_service.js
import {toHyphenatedUuid} from "./uuid_service";

export function parsePostReference(text) {
    if (!text.startsWith('@')) return null;

    let uuid = parseUuid(text.slice(1));
    if (!uuid) return null;
    uuid.match = toHyphenatedUuid(uuid.match);
    return uuid;
}

export function parseUuid(text) {
    const simpleUuid = parseSimpleUuid(text);
    if (simpleUuid) return simpleUuid;

    const hyphenatedUuid = parseHyphenatedUuid(text);
    if (hyphenatedUuid) return hyphenatedUuid;

    return null;
}

export function parseSimpleUuid(text) {
    const simpleUuidRe = /^[\da-f]{8}[\da-f]{4}4[\da-f]{3}[89ab][\da-f]{3}[\da-f]{12}/i;
    return parseByRe(simpleUuidRe, text);
}

export function parseHyphenatedUuid(text) {
    const hyphenatedUuidRe = /^[\da-f]{8}-[\da-f]{4}-4[\da-f]{3}-[89ab][\da-f]{3}-[\da-f]{12}/i;
    return parseByRe(hyphenatedUuidRe, text);
}

const shortcutUuidRe = /^[\da-f]{5}$/;

function parseByRe(re, text) {
    const match = re.exec(text);
    if (!match) return null;
    return { match: match[0], rest: text.slice(match[0].length)}
}
