// file: url_service.js
import {config} from "../config";

const prefix = config.basePath;
const api = `s_api`;

export function getRecentPostsApiUrl() {
    return `/${api}/posts?${toQuery({
        includeThread: true,
        includeBoard: true,
        sortByDate: "desc",
    })}`;
}

export function getPostsBeforeApiUrl(postId) {
    return `/${api}/posts?${toQuery({
        includeThread: true,
        includeBoard: true,
        sortByDate: "desc",
        beforePost: postId,
    })}`;
}

export function getPostsAfterApiUrl(postId) {
    return `/${api}/posts?${toQuery({
        includeThread: true,
        includeBoard: true,
        sortByDate: "asc",
        afterPost: postId,
    })}`;
}

export function getLogsPageApiUrl(pageNumber) {
    return `/${api}/posts?${toQuery({
        includeThread: true,
        includeBoard: true,
        includeTotal: true,
        sortByDate: "desc",
        skip: pageNumber * config.pageSize.general,
    })}`
}

export function getPostApiUrl(postId, includeIndex) {
    return `/${api}/posts/${postId}?${toQuery({
        includeThread: true,
        includeBoard: true,
        includeIndex,
    })}`;
}

export function getBoardUrl(boardName) {
    return `${prefix}/${boardName}/`;
}

export function getThreadUrl(boardName, threadId, pageNumber = null, postId = null) {
    const path = `${prefix}/${boardName}/${threadId}/`;

    const pathAndQuery = pageNumber === null ? path : `${path}?${toQuery({p: pageNumber})}`;

    return postId === null ? pathAndQuery : `${pathAndQuery}#p${postId}`;
}

export function getWarpUrl(boardName, pageNumber) {
    return prefix + (
        (boardName ? `/${boardName}/` : `/chan`) +
        (pageNumber ? `?${toQuery({p: pageNumber})}` : ``)
    );
}

export function getLogsUrl(pageNumber) {
    return `${prefix}/logs?${toQuery({p: pageNumber})}`
}

export function getHomeUrl() {
    return `${prefix}/`;
}

export function getChanUrl() {
    return `${prefix}/chan`
}

export function getPostLinkUrl(postId) {
    return `${prefix}/@${postId}`;
}

export function getThreadPageApiUrl(threadId, pageNumber) {
    // null means the last page
    // + 1 because we want to skip OP

    return `/${api}/threadSlices/${threadId}?${toQuery({
        skip: (pageNumber === null ? 0 : pageNumber * config.pageSize.general) + 1,
        lastSlice: pageNumber === null,
        includeBoard: true,
    })}`;
}

export function getWarpPageApiUrl(boardName, pageNumber) {
    return (`/${api}/warpSlices?${toQuery({
        board: boardName,
        skip: pageNumber * config.pageSize.warp,
        includeBoard: !boardName,
    })}`);
}

export function getPublishApiUrl() {
    return `/${api}/posts`;
}

export function getUpdateHiddenByOpApiUrl() {
    return `/${api}/posts`;
}

export function getUpdateBoardDescriptionApiUrl(boardName) {
    return `/${api}/boards`
}

export function getSubscribeApiUrl() {
    return `/${api}/subscribe`;
}

export function getContinueSubscriptionApiUrl(subscriptionId) {
    return `/${api}/continueSubscription/${subscriptionId}`;
}

export function getAuthorizeApiUrl() {
    return `/${api}/authorize`;
}

export function getCreateUserApiUrl() {
    return `/${api}/users`;
}

export function getServerInfoApiUrl() {
    return `/${api}/serverInfo`;
}

export function toQuery(obj) {
    return Object.keys(obj)
        .filter(k => obj[k] !== null && obj[k] !== undefined)
        .map(k => `${encodeURIComponent(k)}=${encodeURIComponent(obj[k])}`).join('&');
}
