window.$ = (selector) => document.querySelector(selector);
window.$a = (selector) => document.querySelectorAll(selector);
Element.prototype.$ = function (selector) {
    return this.querySelector(selector);
};
Element.prototype.$a = function (selector) {
    return this.querySelectorAll(selector);
};

HTMLCollection.prototype.forEach = function (fn) {
    return Array.prototype.forEach.call(this, fn);
};

HTMLCollection.prototype.indexOf = function (element) {
    return Array.prototype.indexOf.call(this, element);
};

Element.prototype.getClosestByClassName = function (className) {
    for (let parent = this; parent; parent = parent.parentElement) {
        if (parent.classList.contains(className)) {
            return parent;
        }
    }
    return null;
};

Element.prototype.insertBrBefore = function (element) {
    const br = document.createElement("br");
    this.insertBefore(br, element);
    return br;
};

Element.prototype.insertTextBefore = function (text, element) {
    this.insertBefore(document.createTextNode(text), element);
};

Element.prototype.appendText = function (x) {
    const text = x.toString();
    if (text) this.appendChild(document.createTextNode(text));
};

Element.prototype.appendAny = function (x) {
    if (x instanceof Element) {
        this.appendChild(x);
    } else if (typeof x === 'string') {
        this.appendText(x);
    } else if (Array.isArray(x)) {
        x.forEach(item => this.appendAny(item));
    } else {
        console.error(this, x);
        throw Error("appendAny: invalid argument");
    }
    return this;
};

Element.prototype.appendBefore = function (x) {
    x.parentNode.insertBefore(this, x);
};

Element.prototype.appendAfter = function (x) {
    x.parentNode.insertBefore(this, x.nextSibling);
};

DocumentFragment.prototype.appendAfter = function (x) {
    x.parentNode.insertBefore(this, x.nextSibling);
};

Element.prototype.scrollToBottom = function () {
    this.scrollTop = this.scrollHeight;
};

Element.prototype.hide = function () {
    this.classList.add('hidden');
};

Element.prototype.show = function () {
    this.classList.remove('hidden');
};

Element.prototype.isHidden = function () {
    return this.classList.contains('hidden');
};

Array.prototype.last = function () {
    return this[this.length - 1];
};

Number.prototype.getNthByte = function (n) {
    const shift = (n - 1) * 8;
    return (this >> shift) % 0x100;
};

Number.prototype.toRGB = function () {
    return `rgb(${this.getNthByte(3)}, ${this.getNthByte(2)}, ${this.getNthByte(1)})`;
};

Number.prototype.toHSL = function () {
    let h = (this >> 16) % 360;
    let s = this.getNthByte(2) % 101;
    let l = this.getNthByte(1) % 101;
    return `hsl(${h}, ${s}%, ${l}%)`;
};

Promise.prototype.thenA = function (onFulfilled, onRejected) {
    return this.then(f => f(onFulfilled), onRejected);
};

Date.prototype.withoutTime = function () {
    const d = new Date(this);
    d.setHours(0, 0, 0, 0);
    return d;
};

export function promiseTimeout(promise, timeout, timeoutCallback) {
    const timeoutId = setTimeout(timeoutCallback, timeout);

    return promise.then(result => {
        clearTimeout(timeoutId); // prevent calling timeoutCallback
        return result;
    }).catch(e => {
        clearTimeout(timeoutId);
        throw e;
    });
}

export function tryDecodeURI(encodedURI) {
    try {
        return decodeURI(encodedURI);
    } catch (e) {
        return encodedURI;
    }
}

export function importScript(src) {
    return new Promise((resolve, reject) => {
        const scriptE = document.createElement("script");
        scriptE.src = src;
        scriptE.async = true;
        scriptE.onload = resolve;
        scriptE.onerror = reject;
        document.head.appendChild(scriptE);
    });
}

export function importCss(src) {
    return new Promise((resolve, reject) => {
        const linkE = document.createElement("link");
        linkE.rel = "stylesheet";
        linkE.href = src;
        linkE.onload = resolve;
        linkE.onerror = reject;
        document.head.appendChild(linkE);
    });
}

export function formatDate(date) {
    return date.toISOString().slice(0, 19).replace('T', ' ');
}

export function formatTime(date) {
    return formatDate(date).split(' ')[1];
}

export function addVisibilityListener(element, listener) {
    const options = {
        root: document.documentElement,
    };

    let wasHidden = element.isHidden();

    const observer = new IntersectionObserver((entries, observer) => entries.forEach(entry => {
        const isHidden = entry.intersectionRatio <= 0;

        if (isHidden === wasHidden) {
            // nothing changed
            return;
        }

        wasHidden = isHidden;
        listener(isHidden);
    }), options);

    observer.observe(element);
}

export function decorate(srcObj, dstObj) {
    const dstObjPrototype = Object.getPrototypeOf(dstObj);

    for (const property in srcObj) {
        if (!srcObj.hasOwnProperty(property)) continue;

        // ignore getters of dstObj
        const descriptor = Object.getOwnPropertyDescriptor(dstObjPrototype, property);
        const isGetter = descriptor && descriptor.get;

        if (!isGetter && !property.startsWith('_') && !dstObj[property]) {
            // console.log(property, srcObj[property]);
            dstObj[property] = srcObj[property];
        }
    }
    // copy methods
    Object.getOwnPropertyNames(
        Object.getPrototypeOf(srcObj)
    ).forEach(property => {
        if (property !== "constructor" && !property.startsWith('_') && !dstObj[property]) {
            // console.log(property, srcObj[property]);
            dstObj[property] = (...args) => srcObj[property](...args);
        }
    });
}

export function decorateView(scrView, dstView, viewName) {
    decorate(scrView, dstView);

    dstView.element = scrView.element;
    dstView.element[viewName] = dstView;
}

export function parseCssDimension(str) {
    return str === "" ? 0 : parseFloat(str.slice(0, -2));
}
